#pragma once
#ifndef SOFTWARE_VERSION_H
#define SOFTWARE_VERSION_H

#include <string>
#include <map>
#include <boost/algorithm/string.hpp>

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#ifdef major
#undef major
#endif

#ifdef minor
#undef minor
#endif

enum enum_software_version_type
{
    svt_unknown                      = 255,
    svt_pze                          = 0,
    svt_zk                           = 1,
    svt_bde                          = 2,
    svt_mobil                        = 3,
    svt_mde                          = 4,
    svt_ae                           = 5,
    svt_timeboy                      = 6,
    svt_exklusive                    = 7,
    svt_flex                         = 8,
    svt_timeboy_mobil_pze            = 9,
    svt_evo_2_8_pure                 = 10,
    svt_evo_4_3                      = 11,
    svt_extension_a                  = 12,
    svt_extension_b                  = 13,
    svt_extension_c                  = 14,
    svt_io_box                       = 15,
    svt_kyo_inloc                    = 16,
    svt_mobil_box                    = 17,
    svt_timeboy_tbdock               = 18,
    svt_mobil_tbdock                 = 19,
    svt_kyo_oneloc                   = 20,
    svt_evo_3_5_universal            = 21,
    svt_sams_ocl_reader              = 22,
    svt_evo_3_5_pure                 = 23,
    svt_evo_agera                    = 24,
    svt_kyo_cenloc                   = 25,
    svt_vario_10                     = 26,
    svt_vario_12                     = 27,
    svt_vario_15                     = 28,
    svt_vario_17                     = 29,
    svt_ipc_evo_12_1                 = 30,
    svt_ipc_evo_15                   = 31,
    svt_ipc_evo_18_5                 = 32,
    svt_ipc_evo_24                   = 33,
    svt_evo_intera_2                 = 34,
    svt_evo_4_6_flex_key             = 35,
    svt_evo_5_0_pure                 = 36
    // Die Bezeichnung muss in die Tabelle TypeDeviceNameValue::m_type_device_name_value eingetragen werden.
};

class TypeDeviceNameValue
{
public:
    typedef std::map<std::string, std::pair<std::string, enum_software_version_type> > type_device_name_value;
    typedef std::map<enum_software_version_type, std::string> type_device_long_name;

public:
    TypeDeviceNameValue()
    {
        // Inhalt der Map ist: map<lower(Name), pair<Wert, Name> >
        auto make_device_name_value = [this]( const std::string &name, enum_software_version_type val ) -> std::pair<std::string, std::pair<std::string, enum_software_version_type> >
        {
            return std::make_pair( boost::to_lower_copy( name ), std::make_pair( name, val ) );
        };
        // Liste der Geraetebezeichnung zum Vergleich mittels Kleinbuchstaben gemappt auf die Geraetetypnummer und abgesprochene Namensdarstellung.
        // Die Gertebezeichnung muss aus mindestens zwei Zeichen bestehen.
        m_type_device_name_value.insert( make_device_name_value( "?", svt_unknown ) );
        m_type_device_name_value.insert( make_device_name_value( "PZE", svt_pze ) );
        m_type_device_name_value.insert( make_device_name_value( "ZK", svt_zk ) );
        m_type_device_name_value.insert( make_device_name_value( "BDE", svt_bde ) );
        m_type_device_name_value.insert( make_device_name_value( "MOBIL", svt_mobil ) );
        m_type_device_name_value.insert( make_device_name_value( "MDE", svt_mde ) );
        m_type_device_name_value.insert( make_device_name_value( "AE", svt_ae ) );
        m_type_device_name_value.insert( make_device_name_value( "TIMEBOY", svt_timeboy ) );
        m_type_device_name_value.insert( make_device_name_value( "EXKLUSIV", svt_exklusive ) );
        m_type_device_name_value.insert( make_device_name_value( "Flex", svt_flex ) );
        m_type_device_name_value.insert( make_device_name_value( "TB-MOBIL-PZE", svt_timeboy_mobil_pze ) );
        m_type_device_name_value.insert( make_device_name_value( "Evo28", svt_evo_2_8_pure ) );
        m_type_device_name_value.insert( make_device_name_value( "Evo43", svt_evo_4_3 ) );
        m_type_device_name_value.insert( make_device_name_value( "VARIO", svt_extension_a ) );
        m_type_device_name_value.insert( make_device_name_value( "IPC", svt_extension_b ) );
        m_type_device_name_value.insert( make_device_name_value( "Q7", svt_extension_c ) );
        m_type_device_name_value.insert( make_device_name_value( "IOBOX", svt_io_box ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKBOX", svt_kyo_inloc ) );
        m_type_device_name_value.insert( make_device_name_value( "MOBILBOX", svt_mobil_box ) );
        m_type_device_name_value.insert( make_device_name_value( "DockingV2", svt_timeboy_tbdock ) );
        m_type_device_name_value.insert( make_device_name_value( "LoggerV2", svt_mobil_tbdock ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKKNOTEN", svt_kyo_oneloc ) );
        m_type_device_name_value.insert( make_device_name_value( "EVO35U", svt_evo_3_5_universal ) );
        m_type_device_name_value.insert( make_device_name_value( "SAMS-OCL", svt_sams_ocl_reader ) );
        m_type_device_name_value.insert( make_device_name_value( "EVO35", svt_evo_3_5_pure ) );
        m_type_device_name_value.insert( make_device_name_value( "ZKREADERPLUS", svt_evo_agera ) );
        m_type_device_name_value.insert( make_device_name_value( "25", svt_kyo_cenloc ) );
        m_type_device_name_value.insert( make_device_name_value( "26", svt_vario_10 ) );
        m_type_device_name_value.insert( make_device_name_value( "27", svt_vario_12 ) );
        m_type_device_name_value.insert( make_device_name_value( "28", svt_vario_15 ) );
        m_type_device_name_value.insert( make_device_name_value( "29", svt_vario_17 ) );
        m_type_device_name_value.insert( make_device_name_value( "30", svt_ipc_evo_12_1 ) );
        m_type_device_name_value.insert( make_device_name_value( "31", svt_ipc_evo_15 ) );
        m_type_device_name_value.insert( make_device_name_value( "32", svt_ipc_evo_18_5 ) );
        m_type_device_name_value.insert( make_device_name_value( "33", svt_ipc_evo_24 ) );
        m_type_device_name_value.insert( make_device_name_value( "34", svt_evo_intera_2 ) );
        m_type_device_name_value.insert( make_device_name_value( "35", svt_evo_4_6_flex_key ) );
        m_type_device_name_value.insert( make_device_name_value( "36", svt_evo_5_0_pure ) );

        // Liste der langen Gertenamen
        m_type_device_long_name.insert( std::make_pair( svt_unknown, "Unknown" ) );
        m_type_device_long_name.insert( std::make_pair( svt_pze, "PZE-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_zk, "ZK-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_bde, "BDE-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_mobil, "Mobil-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_mde, "MDE-BoxIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_ae, "AE-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_timeboy, "TimeboyIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_exklusive, "Exklusiv-Line IV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_flex, "Flex-MasterIV" ) );
        m_type_device_long_name.insert( std::make_pair( svt_timeboy_mobil_pze, "TimeboyIV Mobil-PZE" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_2_8_pure, "EVO 2.8 Pure" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_4_3, "EVO 4.3" ) );
        m_type_device_long_name.insert( std::make_pair( svt_extension_a, "IPC Embedded-System A" ) );
        m_type_device_long_name.insert( std::make_pair( svt_extension_b, "IPC Embedded-System B" ) );
        m_type_device_long_name.insert( std::make_pair( svt_extension_c, "IPC Embedded-System C" ) );
        m_type_device_long_name.insert( std::make_pair( svt_io_box, "IO-Box V4" ) );
        m_type_device_long_name.insert( std::make_pair( svt_kyo_inloc, "KYO Inloc" ) );
        m_type_device_long_name.insert( std::make_pair( svt_mobil_box, "Mobil-Box V4" ) );
        m_type_device_long_name.insert( std::make_pair( svt_timeboy_tbdock, "Dockingstation V2" ) );
        m_type_device_long_name.insert( std::make_pair( svt_mobil_tbdock, "Fahrzeugdatenlogger V2" ) );
        m_type_device_long_name.insert( std::make_pair( svt_kyo_oneloc, "KYO Oneloc" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_3_5_universal, "EVO 3.5 Universal" ) );
        m_type_device_long_name.insert( std::make_pair( svt_sams_ocl_reader, "SAMS-OCL-Reader" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_3_5_pure, "EVO 3.5 Pure" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_agera, "EVO Agera" ) );
        m_type_device_long_name.insert( std::make_pair( svt_kyo_cenloc, "KYO Cenloc" ) );
        m_type_device_long_name.insert( std::make_pair( svt_vario_10, "Vario 10" ) );
        m_type_device_long_name.insert( std::make_pair( svt_vario_12, "Vario 12" ) );
        m_type_device_long_name.insert( std::make_pair( svt_vario_15, "Vario 15" ) );
        m_type_device_long_name.insert( std::make_pair( svt_vario_17, "Vario 17" ) );
        m_type_device_long_name.insert( std::make_pair( svt_ipc_evo_12_1, "IPC EVO 12.1" ) );
        m_type_device_long_name.insert( std::make_pair( svt_ipc_evo_15, "IPC EVO 15" ) );
        m_type_device_long_name.insert( std::make_pair( svt_ipc_evo_18_5, "IPC EVO 18.5" ) );
        m_type_device_long_name.insert( std::make_pair( svt_ipc_evo_24, "IPC EVO 24" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_intera_2, "EVO Intera 2" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_4_6_flex_key, "EVO 4.6 FlexKey" ) );
        m_type_device_long_name.insert( std::make_pair( svt_evo_5_0_pure, "EVO 5.0 Pure" ) );
    }

public:
    std::string get_device_name( enum_software_version_type type );
    enum_software_version_type get_device_value( const std::string &str );

    std::string get_device_long_name( enum_software_version_type type );

private:
    type_device_name_value m_type_device_name_value;
    type_device_long_name m_type_device_long_name;
};

class SoftwareVersion
{
public:
    SoftwareVersion();
    SoftwareVersion( const std::string &version );
    SoftwareVersion( const char *version );
    SoftwareVersion( const char *version, const char *branchInfo );
    SoftwareVersion( unsigned int version );
    SoftwareVersion( unsigned char master, char major, unsigned char minor, unsigned char build, enum_software_version_type type_device = svt_unknown );
    SoftwareVersion( unsigned char master, char major, unsigned char minor, unsigned char build, const char *branchInfo );
    virtual ~SoftwareVersion();
    friend class SoftwarebuildVersion;

private:
    std::string m_version;
    std::string m_branchIdent;
    unsigned char m_master;
    unsigned char m_major;
    unsigned char m_minor;
    unsigned char m_build;
    enum_software_version_type m_type_device;

public:
    void reset();
    bool parse( const std::string &version );
    bool parse( unsigned int version );

    inline unsigned char master() const
    {
        return m_master;
    }
    inline unsigned char major() const
    {
        return m_major;
    }
    inline unsigned char minor() const
    {
        return m_minor;
    }
    inline unsigned char build() const
    {
        return m_build;
    }
    inline enum_software_version_type typeDevice()
    {
        return m_type_device;
    }

public: // Zuweisungsoperator
    SoftwareVersion &operator=( const char *version )
    {
        if ( this->parse( version ) == false )
        {
            this->reset();
        }

        return *this;
    }
    SoftwareVersion &operator=( unsigned int version )
    {
        if ( this->parse( version ) == false )
        {
            this->reset();
        }

        return *this;
    }

public: // Vergleichsoperatoren
    // Gleichheit
    bool operator==( SoftwareVersion &version )
    {
        if ( this->m_master == version.m_master
             && this->m_major == version.m_major
             && this->m_minor == version.m_minor )
        {
            return true;
        }

        return false;
    }
    bool operator==( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this == lVersion);
    }
    // Ungleichheit
    bool operator!=( SoftwareVersion &version )
    {
        if ( this->m_master != version.m_master
             || this->m_major != version.m_major
             || this->m_minor != version.m_minor )
        {
            return true;
        }

        return false;
    }
    bool operator!=( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this != lVersion);
    }
    // Groesser
    bool operator>( SoftwareVersion &version )
    {
        if ( this->m_master > version.m_master
             || (this->m_master == version.m_master && this->m_major > version.m_major)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor) )
        {
            return true;
        }

        return false;
    }
    bool operator>( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this > lVersion);
    }
    // Groesser gleich
    bool operator>=( SoftwareVersion &version )
    {
        if ( *this > version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator>=( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this >= lVersion);
    }
    // Kleiner
    bool operator<( SoftwareVersion &version )
    {
        if ( this->m_master < version.m_master
             || (this->m_master == version.m_master && this->m_major < version.m_major)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor) )
        {
            return true;
        }

        return false;
    }
    bool operator<( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this < lVersion);
    }
    // Kleiner gleich
    bool operator<=( SoftwareVersion &version )
    {
        if ( *this < version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator<=( const char *version )
    {
        SoftwareVersion lVersion( version );
        return (*this <= lVersion);
    }

public: // Cast operatoren
    operator const char *()
    {
        return m_version.c_str();
    }
    operator unsigned int()
    {
        return static_cast<unsigned int>((m_master << 24) | (m_major << 16) | (m_minor << 8) | m_build);
    }
};

class SoftwarebuildVersion : public SoftwareVersion
{
public:
    SoftwarebuildVersion() : SoftwareVersion()
    {

    }
    SoftwarebuildVersion( const std::string &version )
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( const char *version )
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( const char *version, const char *branchInfo )
        : SoftwareVersion( version, branchInfo )
    {

    }
    SoftwarebuildVersion( unsigned int version )
        : SoftwareVersion( version )
    {

    }
    SoftwarebuildVersion( unsigned char master, char major, unsigned char minor, unsigned char build, enum_software_version_type type_device = svt_unknown )
        : SoftwareVersion( master, major, minor, build, type_device )
    {
    }
    SoftwarebuildVersion( unsigned char master, char major, unsigned char minor, unsigned char build, const char *branchInfo )
        : SoftwareVersion( master, major, minor, build, branchInfo )
    {
    }

public: // Vergleichsoperatoren
    // Gleichheit
    bool operator==( const SoftwarebuildVersion &version ) const
    {
        if ( this->m_master == version.m_master &&
             this->m_major == version.m_major &&
             this->m_minor == version.m_minor &&
             this->m_build == version.m_build )
        {
            return true;
        }
        return false;
    }
    bool operator==( const char *version ) const
    {
        SoftwarebuildVersion lVersion( version );
        return (*this == lVersion);
    }

    // Ungleichheit
    bool operator!=( const SoftwarebuildVersion &version ) const
    {
        return !(*this == version);
    }
    bool operator!=( const char *version ) const
    {
        return !(*this == version);
    }

    // Groesser
    bool operator>( const SoftwarebuildVersion &version ) const
    {
        if ( this->m_master > version.m_master
             || (this->m_master == version.m_master && this->m_major > version.m_major)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build > version.m_build) )
        {
            return true;
        }

        return false;
    }
    bool operator>( const char* version ) const
    {
        SoftwarebuildVersion lVersion( version );
        return (*this > lVersion);
    }
    // Groesser gleich
    bool operator>=( const SoftwarebuildVersion& version ) const
    {
        if ( *this > version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator>=( const char* version ) const
    {
        SoftwarebuildVersion lVersion( version );
        return (*this >= lVersion);
    }
    // Kleiner
    bool operator<( const SoftwarebuildVersion& version ) const
    {
        if ( this->m_master < version.m_master
             || (this->m_master == version.m_master && this->m_major < version.m_major)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor)
             || (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build < version.m_build) )
        {
            return true;
        }

        return false;
    }
    bool operator<( const char* version ) const
    {
        SoftwarebuildVersion lVersion( version );
        return (*this < lVersion);
    }
    // Kleiner gleich
    bool operator<=( const SoftwarebuildVersion& version ) const
    {
        if ( *this < version || *this == version )
        {
            return true;
        }

        return false;
    }
    bool operator<=( const char* version ) const
    {
        SoftwarebuildVersion lVersion( version );
        return (*this <= lVersion);
    }

};
#endif
